@@ -21,11 +21,18 @@
 #define FIFTEEN_H
 
 #include <QGraphicsWidget>
+#include <QHash>
+#include <QWeakPointer>
 
 #include <Plasma/Svg>
 
 class Piece;
 
+namespace Plasma
+{
+    class Animation;
+}
+
 class Fifteen : public QGraphicsWidget
 {
   Q_OBJECT
@@ -62,6 +69,7 @@ class Fifteen : public QGraphicsWidget
     QFont m_font;
     bool m_splitPixmap;
     bool m_numerals;
+    QHash<Piece*, QWeakPointer<Plasma::Animation> > m_animations;
 };
 
 #endif
