@@ -23,10 +23,12 @@
 #define FRAME_H
 
 #include <QAction>
+#include <QImage>
 #include <QPixmap>
 #include <QPainter>
 #include <QGraphicsItem>
 #include <QColor>
+#include <QTimer>
 
 #include <plasma/applet.h>
 #include <plasma/animator.h>
@@ -70,6 +72,10 @@ private Q_SLOTS:
     void removeDir();
     void updateButtons();
     void delayedUpdateSize();
+    void scalePictureAndUpdate();
+    void imageScaled(const QImage &img);
+    void reloadImage();
+    void checkDayChanged();
 
 protected:
     void constraintsEvent(Plasma::Constraints constraints);
@@ -90,8 +96,6 @@ private:
     bool m_frame;
     /// If true, the frame will have rounded corners
     bool m_roundCorners;
-    /// If true, smooth scaling (better visual results, but more cpu intensive) is used for resizing
-    bool m_smoothScaling;
     /// If true, the picture will have a drop shadow.
     bool m_shadow;
     /// PoTD
@@ -114,11 +118,21 @@ private:
     bool m_random;
     bool m_recursiveSlideShow;
     SlideShow* m_mySlideShow;
+    /// Auto update
+    int m_autoUpdateIntervall;
+    bool m_doAutoUpdate;
+    QTimer* m_autoUpdateTimer;
+    /// Day Change for PoTD
+    QTimer *m_dateChangedTimer;
+    QDate m_currentDay;
+
     QPixmap m_pixmap;
+    QImage m_scaledImage;
     QTimer* m_updateTimer;
     Plasma::ToolButton* m_backButton;
     Plasma::ToolButton* m_nextButton;
     Plasma::Frame* m_slideFrame;
+    QTimer m_waitForResize;
 };
 
 K_EXPORT_PLASMA_APPLET(frame, Frame)
